import FormBuilder from './FormBuilder.vue';
import { CustomJSONSchema } from './type/custom-json-schema';
import { Columns } from './utils/columns.utils';
import { IFormWidget } from './widgets/FormWidget';
export type FormConfig<T> = T extends number ? number : object;
export type ValidatedMeta = {
    validity: boolean;
    path: string;
    err?: { msg: string }[];
};

export type FormSchema<T extends { properties?; widget: IWidget; items? } = any> = {
    label?: string | false;
    items?: FormSchema<T['items']>;
    properties?: {
        [Key in keyof T['properties']]: FormSchema<T['properties'][Key]>;
    };
    widget: IWidget;
    validate?: boolean | ((context: IFormWidget) => boolean);
    validateFn?: (value: any, context: IFormWidget) => ValidatedMeta;
    targetPath?: string;
    view?: boolean;
    hidden?: boolean | ((context: IFormWidget) => boolean);
    required?: boolean | ((context: IFormWidget) => boolean);
    columns?: Columns;
    //listenTo?: string[];
    defaultValue?: ConstructorToType<T['widget']['valueType']>;
    config?: T['widget']['config'];
    [key: string]: any;
};
export interface IWidget<C = any, T = any, V = any> {
    template: T;
    columns: Columns;
    config?: C;
    valueType?: V;
    jsonSchemaConverter?: (formSchema: FormSchema, jsonSchema: CustomJSONSchema) => void;
}
export type FormBuilderInstance = InstanceType<typeof FormBuilder>;
export interface FormBuilderOption {
    save?: { location: string; autoSave: boolean } | false;
}
export interface FormHistory {
    value: any;
    expiryAt: number;
}
export type ConstructorToType<T extends new (...args: any[]) => any> = T extends new (...args: any[]) => infer R ? R : never;

export interface FormBuilderProps {
    schema: FormSchema;
    view?: boolean;
    disabled?: boolean;
    value?: any;
    saveLocation?: string;
    saveExpiry?: number;
    saveDelay?: number;
    autoSave?: boolean;
}
export interface ExportValueOptions {
    removeAdditional?: boolean;
}
