import { defineAsyncComponent, markRaw } from 'vue';
import { FormSchema, IWidget } from '../FormBuilder.interface';
import { ArrayWidgetConfig } from './ArrayWidget.config';
import { BooleanWidgetConfig } from './BooleanWidget.config';
import { DateWidgetConfig } from './DateWidget.config';
import { NumberWidgetConfig } from './NumberWidget.config';
import { PasswordWidgetConfig } from './PasswordWidget.config';
import { StringWidgetConfig } from './StringWidget.config';
import { TextAreaWidgetConfig } from './TextAreaWidget.config';
import { deepAssign } from '@codeperate/utils';
export function defineWidget<C, T, V>(widget: IWidget<C, T, V>) {
    return widget;
}

export const StringWidget = defineWidget({
    template: markRaw(defineAsyncComponent(() => import('./StringWidget.vue'))),
    columns: 6,
    valueType: String,
    config: {} as Partial<StringWidgetConfig>,
});

export const ObjectWidget = defineWidget({
    template: markRaw(defineAsyncComponent(() => import('./ObjectWidget.vue'))),
    columns: 12,
    valueType: Object,
});

export const TextAreaWidget = defineWidget({
    template: markRaw(defineAsyncComponent(() => import('./TextAreaWidget.vue'))),
    columns: 12,
    valueType: String,
    config: {} as Partial<TextAreaWidgetConfig>,
});

export const PasswordWidget = defineWidget({
    template: markRaw(defineAsyncComponent(() => import('./PasswordWidget.vue'))),
    columns: 12,
    valueType: String,
    config: {} as Partial<PasswordWidgetConfig>,
});

export const ArrayWidget = defineWidget({
    template: markRaw(defineAsyncComponent(() => import('./ArrayWidget.vue'))),
    columns: 12,
    valueType: Array,
    config: {} as Partial<ArrayWidgetConfig>,
});

export const NumberWidget = defineWidget({
    template: markRaw(defineAsyncComponent(() => import('./NumberWidget.vue'))),
    columns: 6,
    valueType: Number,
    config: {} as Partial<NumberWidgetConfig>,
});

export const BooleanWidget = defineWidget({
    template: markRaw(defineAsyncComponent(() => import('./BooleanWidget.vue'))),
    columns: 6,
    valueType: Boolean,
    config: {} as Partial<BooleanWidgetConfig>,
});

export const DateWidget = defineWidget({
    template: markRaw(defineAsyncComponent(() => import('./DateWidget.vue'))),
    columns: 6,
    valueType: String,
    config: {} as Partial<DateWidgetConfig>,
});

export const DateTimeWidget = defineWidget({
    template: markRaw(defineAsyncComponent(() => import('./DateTimeWidget.vue'))),
    columns: 6,
    valueType: String,
    config: {} as Partial<ArrayWidgetConfig>,
});

export const SectionWidget: IWidget<{ title: string }> = {
    template: markRaw(defineAsyncComponent(() => import('./SectionWidget.vue'))),
    columns: 12,
};
export const Section = (title: string, formSchema: Partial<FormSchema> = {}, key?: number | string | symbol) => {
    const obj = deepAssign({ widget: SectionWidget, config: { title }, label: false }, formSchema);
    return { [key ?? title]: obj };
};
